@@ -20,6 +20,7 @@
 
 #include "anim/animatable.h"
 #include "bling.h"
+#include "cursor.h"
 #include "cutouts-overlay.h"
 #include "settings.h"
 #include "layer-shell.h"
@@ -126,19 +127,21 @@ phoc_output_frame_callback_info_free (PhocOutputFrameCallbackInfo *cb_info)
  */
 static bool
 get_surface_box (PhocOutputSurfaceIteratorData *data,
-                 struct wlr_surface *surface, int sx, int sy,
-                 struct wlr_box *surface_box)
+                 struct wlr_surface            *wlr_surface,
+                 int                            sx,
+                 int                            sy,
+                 struct wlr_box                *surface_box)
 {
   PhocOutput *self = data->output;
 
-  if (!wlr_surface_has_buffer (surface))
+  if (!wlr_surface_has_buffer (wlr_surface))
     return false;
 
   struct wlr_box box = {
     .x = floor (data->ox + sx),
     .y = floor (data->oy + sy),
-    .width = surface->current.width,
-    .height = surface->current.height,
+    .width = wlr_surface->current.width,
+    .height = wlr_surface->current.height,
   };
 
   if (surface_box != NULL)
@@ -348,20 +351,20 @@ phoc_output_set_gamma_lut (PhocOutput *self, struct wlr_output_state *pending)
 
 static void
 surface_send_frame_done_iterator (PhocOutput         *output,
-                                  struct wlr_surface *surface,
+                                  struct wlr_surface *wlr_surface,
                                   struct wlr_box     *box,
                                   float               scale,
                                   void               *data)
 {
   struct timespec *when = data;
 
-  wlr_surface_send_frame_done (surface, when);
+  wlr_surface_send_frame_done (wlr_surface, when);
 }
 
 
 static void
 count_surface_iterator (PhocOutput         *output,
-                        struct wlr_surface *surface,
+                        struct wlr_surface *wlr_surface,
                         struct wlr_box     *box,
                         float               scale,
                         void               *data)
@@ -393,6 +396,9 @@ scan_out_fullscreen_view (PhocOutput *self, PhocView *view, struct wlr_output_st
       return false;
   }
 
+  if (phoc_output_has_shell_revealed (self))
+    return false;
+
   if (phoc_output_has_layer (self, ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY))
     return false;
 
@@ -594,14 +600,14 @@ phoc_output_handle_needs_frame (struct wl_listener *listener, void *user_data)
 
 static void
 update_output_scale_iterator (PhocOutput         *self,
-                              struct wlr_surface *surface,
+                              struct wlr_surface *wlr_surface,
                               struct wlr_box     *box,
                               float               scale,
                               void               *user_data)
 
 
 {
-  phoc_utils_wlr_surface_update_scales (surface);
+  phoc_utils_wlr_surface_update_scales (wlr_surface);
 }
 
 
@@ -665,7 +671,7 @@ phoc_output_compute_scale (PhocOutput *self, struct wlr_output_state *pending)
     return 1;
   }
 
-  // Use the pending mode if any
+  /*  Use the pending mode if any */
   if (pending->committed & WLR_OUTPUT_STATE_MODE) {
     switch (pending->mode_type) {
     case WLR_OUTPUT_STATE_MODE_FIXED:
@@ -679,7 +685,7 @@ phoc_output_compute_scale (PhocOutput *self, struct wlr_output_state *pending)
     default:
       break;
     }
-  // Fall back to current mode
+  /* Fall back to current mode */
   } else if (self->wlr_output->current_mode) {
     width = self->wlr_output->current_mode->width;
     height = self->wlr_output->current_mode->height;
@@ -702,7 +708,7 @@ phoc_output_state_set_mode (PhocOutput *self, struct wlr_output_state *pending,
   int mhz = (int)(oc->mode.refresh_rate * 1000);
 
   if (wl_list_empty (&self->wlr_output->modes)) {
-    // Output has no mode, try setting a custom one
+    /* Output has no mode, try setting a custom one */
     wlr_output_state_set_custom_mode (pending, oc->mode.width, oc->mode.height, mhz);
     return;
   }
@@ -901,7 +907,7 @@ phoc_output_initable_init (GInitable    *initable,
 
     g_assert (PHOC_IS_SEAT (seat));
     phoc_seat_configure_cursor (seat);
-    phoc_seat_configure_xcursor (seat);
+    phoc_cursor_configure_xcursor (seat->cursor);
   }
 
   phoc_layer_shell_arrange (self);
@@ -1020,7 +1026,7 @@ phoc_output_class_init (PhocOutputClass *klass)
 }
 
 static void
-phoc_output_for_each_surface_iterator (struct wlr_surface *surface,
+phoc_output_for_each_surface_iterator (struct wlr_surface *wlr_surface,
                                        int                 sx,
                                        int                 sy,
                                        void               *_data)
@@ -1028,13 +1034,13 @@ phoc_output_for_each_surface_iterator (struct wlr_surface *surface,
   PhocOutputSurfaceIteratorData *data = _data;
 
   struct wlr_box box;
-  bool intersects = get_surface_box (data, surface, sx, sy, &box);
+  bool intersects = get_surface_box (data, wlr_surface, sx, sy, &box);
 
   if (!intersects) {
     return;
   }
 
-  data->user_iterator (data->output, surface, &box, data->scale, data->user_data);
+  data->user_iterator (data->output, wlr_surface, &box, data->scale, data->user_data);
 }
 
 /**
@@ -1050,7 +1056,7 @@ phoc_output_for_each_surface_iterator (struct wlr_surface *surface,
  */
 void
 phoc_output_surface_for_each_surface (PhocOutput          *self,
-                                      struct wlr_surface  *surface,
+                                      struct wlr_surface  *wlr_surface,
                                       double               ox,
                                       double               oy,
                                       PhocSurfaceIterator  iterator,
@@ -1062,12 +1068,12 @@ phoc_output_surface_for_each_surface (PhocOutput          *self,
     .output = self,
     .ox = ox,
     .oy = oy,
-    .width = surface->current.width,
-    .height = surface->current.height,
+    .width = wlr_surface->current.width,
+    .height = wlr_surface->current.height,
     .scale = 1.0
   };
 
-  wlr_surface_for_each_surface (surface,
+  wlr_surface_for_each_surface (wlr_surface,
                                 phoc_output_for_each_surface_iterator, &data);
 }
 
@@ -1165,7 +1171,7 @@ phoc_output_xwayland_children_for_each_surface (PhocOutput                  *sel
   struct wlr_xwayland_surface *child;
 
   wl_list_for_each (child, &surface->children, parent_link) {
-    if (child->surface->mapped) {
+    if (child->surface && child->surface->mapped) {
       double ox = child->x - output_box.x;
       double oy = child->y - output_box.y;
       phoc_output_surface_for_each_surface (self, child->surface, ox, oy, iterator,
@@ -1479,7 +1485,7 @@ phoc_view_accept_damage (PhocOutput *self, PhocView  *view)
 
 #ifdef PHOC_XWAYLAND
   if (PHOC_IS_XWAYLAND_SURFACE (self->fullscreen_view) && PHOC_IS_XWAYLAND_SURFACE (view)) {
-    // Special case: accept damage from children
+    /* Special case: accept damage from children */
     struct wlr_xwayland_surface *xsurface =
       phoc_xwayland_surface_get_wlr_surface (PHOC_XWAYLAND_SURFACE (view));
     struct wlr_xwayland_surface *fullscreen_xsurface =
@@ -1496,7 +1502,7 @@ phoc_view_accept_damage (PhocOutput *self, PhocView  *view)
 }
 
 static void
-damage_surface_iterator (PhocOutput *self, struct wlr_surface *surface, struct wlr_box *_box,
+damage_surface_iterator (PhocOutput *self, struct wlr_surface *wlr_surface, struct wlr_box *_box,
                          float scale, void *data)
 {
   bool *whole = data;
@@ -1508,13 +1514,13 @@ damage_surface_iterator (PhocOutput *self, struct wlr_surface *surface, struct w
 
   pixman_region32_t damage;
   pixman_region32_init (&damage);
-  wlr_surface_get_effective_damage (surface, &damage);
+  wlr_surface_get_effective_damage (wlr_surface, &damage);
   wlr_region_scale (&damage, &damage, scale);
   wlr_region_scale (&damage, &damage, self->wlr_output->scale);
-  if (ceil (self->wlr_output->scale) > surface->current.scale) {
-    // When scaling up a surface, it'll become blurry so we need to
-    // expand the damage region
-    wlr_region_expand (&damage, &damage, ceil (self->wlr_output->scale) - surface->current.scale);
+  if (ceil (self->wlr_output->scale) > wlr_surface->current.scale) {
+    /* When scaling up a surface, it'll become blurry so we need to
+     * expand the damage region */
+    wlr_region_expand (&damage, &damage, ceil (self->wlr_output->scale) - wlr_surface->current.scale);
   }
 
   pixman_region32_translate (&damage, box.x, box.y);
@@ -1527,7 +1533,7 @@ damage_surface_iterator (PhocOutput *self, struct wlr_surface *surface, struct w
       wlr_output_schedule_frame (self->wlr_output);
   }
 
-  if (!wl_list_empty (&surface->current.frame_callback_list))
+  if (!wl_list_empty (&wlr_surface->current.frame_callback_list))
     wlr_output_schedule_frame (self->wlr_output);
 }
 
@@ -1611,13 +1617,13 @@ phoc_output_damage_whole_surface (PhocOutput         *self,
 
 void
 phoc_output_damage_from_surface (PhocOutput         *self,
-                                 struct wlr_surface *surface,
+                                 struct wlr_surface *wlr_surface,
                                  double              ox,
                                  double              oy)
 {
   bool whole = false;
 
-  phoc_output_surface_for_each_surface (self, surface, ox, oy,
+  phoc_output_surface_for_each_surface (self, wlr_surface, ox, oy,
                                         damage_surface_iterator, &whole);
 }
 
@@ -2092,9 +2098,15 @@ void
 phoc_output_force_shell_reveal (PhocOutput *self, gboolean force)
 {
   PhocOutputPrivate *priv;
+  PhocServer *server = phoc_server_get_default ();
   g_assert (PHOC_IS_OUTPUT (self));
   priv = phoc_output_get_instance_private (self);
 
+  if (G_UNLIKELY (phoc_server_check_debug_flags (server,
+                                                 PHOC_SERVER_DEBUG_FLAG_FORCE_SHELL_REVEAL))) {
+      force = TRUE;
+  }
+
   priv->force_shell_reveal = force;
   phoc_output_update_shell_reveal (self);
 }
@@ -2187,9 +2199,9 @@ phoc_output_transform_box (PhocOutput *self, struct wlr_box *box)
 
   g_assert (PHOC_IS_OUTPUT (self));
 
-  wlr_output_transformed_resolution(self->wlr_output, &ow, &oh);
+  wlr_output_transformed_resolution (self->wlr_output, &ow, &oh);
   transform = wlr_output_transform_invert (self->wlr_output->transform);
-  wlr_box_transform(box, box, transform, ow, oh);
+  wlr_box_transform (box, box, transform, ow, oh);
 }
 
 
